/****************************************************************************
* keyboard_shortcuts.cpp
*
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include "keyboard_shortcuts.h"
#include <qlabel.h>
#include <qvbox.h>
#include <qlistview.h>
#include <qaction.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qmessagebox.h>

#include <assert.h>

#include <stdio.h>

// class KeyboardAction: public QListViewItem
// {
// public:
//   KeyboardAction( QListView *parent, QAction *action, int key ) :
//     QListViewItem( parent ),
//     {
//     }  
// };

KeyboardShortcuts::KeyboardShortcuts( QWidget *parent )
  : QObject( parent )
{
}


void KeyboardShortcuts::addAction( QAction *action )
{
  actionList.append( action );
}

void KeyboardShortcuts::load( Config *conf )
{
  conf->setGroup("keyboard");

  QListIterator<QAction> it( actionList );
  for( ; it.current() != NULL; ++it ) {
    int key = conf->readNumEntry( (*it)->text(), -1 );
    if( key != -1 )
      (*it)->setAccel( key );
  }
}

void KeyboardShortcuts::save( Config *conf )
{
  conf->setGroup("keyboard");

  QListIterator<QAction> it( actionList );
  for( ; it.current() != NULL; ++it ) {
    conf->writeEntry( (*it)->text(), (*it)->accel() );
  }  

}

// ------- Keyboard Setting Dialog -------

class KeyActionListViewItem: public QListViewItem
{
  QAction *action;
  int keyCode;
public:
  KeyActionListViewItem( QListView *parent, QAction *action )
    : QListViewItem( parent ), action( action )
  {
    keyCode = action->accel();
    updateText();
  }

  QAction *getAction( )
  {
    return action;
  }

  void setKey( int keyCode )
  {
    this->keyCode = keyCode;
    updateText();
  }

  int getKey()
  {
    return keyCode;
  }

  void updateText()
  {
    setText( 1, keyCode ? describeKey( keyCode ) : QString::null );
  }  
};


void KeyboardShortcuts::execConfig()
{
  KeyboardShortcutsDialog settings( (QWidget*)parent(), this );

  if( settings.exec() == QDialog::Accepted ) {
    if( settings.modified ) 
      // Copy keyboard binding to QAction's
      for( KeyActionListViewItem *item = (KeyActionListViewItem*)settings.actionLV->firstChild();
           item != NULL; item = (KeyActionListViewItem*)item->nextSibling() ) {
        QAction *action = item->getAction();
        action->setAccel( item->getKey() );
      }
  } else {
    if( settings.modified ) 
      QMessageBox::information( (QWidget*)parent(), tr( "Settings cancelled" ),
        tr( "Your keyboard settings\nhas been cancelled" ), QMessageBox::Ok );
  }
}

KeyboardShortcutsDialog::KeyboardShortcutsDialog( QWidget *parent, KeyboardShortcuts *ks ) :
  QDialog( parent, "keyboard_shortcuts", true ),
  grabKeyFor( NULL ), ignoreReturn( false ),
  modified( false )
{
  setCaption( tr( "Keyboard Config" ) );

  setMinimumWidth( 200 );
  // -4 since the dialog has a frame
  setMinimumHeight( parentWidget()->height()-4 );
  
  QVBox* vbox = new QVBox( this );
  
  vbox->setSpacing(4);
  vbox->setMargin(4);
  
//   new QLabel( tr( "Select an action, press enter\n"
//                   "and then the key to be attached\n"
//                   "to that action." ), vbox );
  
  actionLV = new QListView( vbox );
  actionLV->addColumn( tr( "Action" ) );
  actionLV->addColumn( tr( "Key" ) );
  actionLV->setSorting( -1 );

  QListIterator<QAction> it( ks->actionList );
  it.toLast();
  for( ; it.current() != NULL; --it ) {
    QAction *qa = *it;
    KeyActionListViewItem *item = new KeyActionListViewItem( actionLV, *it );
    item->setText( 0, qa->text() );
    item->setPixmap( 0, qa->iconSet().pixmap() );
    actionLV->insertItem( item );
  }
//   connect( actionLV, SIGNAL(clicked(QListViewItem*)), 
//     this, SLOT(keyActionClicked(QListViewItem*)));
  connect( actionLV, SIGNAL(returnPressed(QListViewItem*)), 
    this, SLOT(keyActionClicked(QListViewItem*)));

  connect( actionLV, SIGNAL(currentChanged(QListViewItem*)), 
    this, SLOT(cancelSetKey()));
  
  QHBox* hbox = new QHBox( vbox );
  QButton *removeKey = new QPushButton( tr( "Clear key" ), hbox );  
  QButton *setKey = new QPushButton( tr( "Set key" ), hbox );
  connect( removeKey, SIGNAL(clicked()), 
    this, SLOT(removeKey()));
  connect( setKey, SIGNAL(clicked()), 
    this, SLOT(setKey()));

  
  QBoxLayout * l = new QVBoxLayout( this );
  l->addWidget( vbox );
}

void KeyboardShortcutsDialog::setKey()
{
  KeyActionListViewItem *item = (KeyActionListViewItem*)actionLV->currentItem();
  if( item == NULL ) return;
  item->setText( 1, "<press key>" );
  grabKeyFor = item;
  grabKeyboard();  
}

void KeyboardShortcutsDialog::removeKey()
{
  cancelSetKey();
  
  KeyActionListViewItem *item = (KeyActionListViewItem*)actionLV->currentItem();
  if( item == NULL ) return;
  item->setKey( 0 );
  modified = true;
}

void KeyboardShortcutsDialog::cancelSetKey()
{
  if( grabKeyFor != NULL ) {
    KeyActionListViewItem *ka = (KeyActionListViewItem*)grabKeyFor;
    ka->updateText();
    grabKeyFor = NULL;
    releaseKeyboard();
  }
}

void KeyboardShortcutsDialog::keyActionClicked( QListViewItem *item )
{
  ignoreReturn = true;
  setKey();
}

void KeyboardShortcutsDialog::keyPressEvent( QKeyEvent *e )
{
  if( grabKeyFor != NULL ) {
    switch (e->key())
    {
    case Key_Shift:
    case Key_Control:
    case Key_Meta:
    case Key_Alt:
    case Key_CapsLock:
    case Key_NumLock:
    case Key_ScrollLock:
    case Key_F22:
      e->ignore();
      break;
    default:
      if( ignoreReturn && e->key() == Key_Return ) {
        e->ignore();
        break;
      }
//      printf( "key: %x modif: %x\n", e->key(), e->state() );
      
      KeyActionListViewItem *ka = (KeyActionListViewItem*)grabKeyFor;

      int keyCode = e->key();
      if( (e->state() & ShiftButton) != 0 )
        keyCode |= SHIFT;
      if( (e->state() & AltButton) != 0 )
        keyCode |= ALT;
      if( (e->state() & ControlButton) != 0 )
        keyCode |= CTRL;      

      //    printf( "keycode: %x\n", keyCode );
      releaseKeyboard();

      // Check for duplicate key codes
      bool cancelBinding = false;
      for( KeyActionListViewItem *item = (KeyActionListViewItem*)actionLV->firstChild();
           item != NULL; item = (KeyActionListViewItem*)item->nextSibling() ) {
        if( item == ka ) continue;
        
        if( item->getKey() == keyCode ) {
          int sel = QMessageBox::warning( this, tr( "Duplicate key binding" ),
            tr( "Key '" ) + describeKey( keyCode ) + tr( "'\nis currently assigned to\n'" ) + item->getAction()->text() +
            tr( "'.\nOverwrite the current assignment?" ), QMessageBox::No, QMessageBox::Yes );
          if( sel == QMessageBox::Yes ) 
            item->setKey( 0 );
          else {
            cancelBinding = true;
            break;
          }
        }
      }

      if( cancelBinding ) 
        ka->updateText();
      else {
        ka->setKey( keyCode );
        modified = true;
      }
      
      grabKeyFor = NULL;
      e->accept();
      break;
    }

  }
  ignoreReturn = false;
  
  QDialog::keyPressEvent( e );
}

QString describeKey( int key )
{
  QString ks;
  if( (key & Qt::CTRL) != 0 ) {
      ks += "Ctrl+";
  }
  if( (key & Qt::ALT) != 0 ) {
      ks += "Alt+";
  }
  if( (key & Qt::SHIFT) != 0) {
    ks += "Shift+";
  }

  key = key & ~Qt::MODIFIER_MASK;
  
  if (key == 0x20)
    {
      ks += "Space";
    }
  else if (0x21 <= key && key <= 0xff)
    {
      char st[2];
      st[0] = key;
      st[1] = 0;
      ks += st;
    }
  else if (0x1030 <= key && key <= 0x1052)
    {
      ks += QString("f") + QChar(key - 0x1030 + 1);
    }
  else
    {
      switch (key)
	{
	case Qt::Key_Escape:
	  ks += "Esc";
	  break;
	case Qt::Key_Tab:
	  ks += "Tab";
	  break;
	case Qt::Key_Backtab:
	  ks += "Backtab";
	  break;
	case Qt::Key_Backspace:
	  ks += "Backspace";
	  break;
	case Qt::Key_Return:
	  ks += "Return";
	  break;
	case Qt::Key_Enter:
	  ks += "Enter";
	  break;
	case Qt::Key_Insert:
	  ks += "Insert";
	  break;
	case Qt::Key_Delete:
	  ks += "Delete";
	  break;
	case Qt::Key_Pause:
	  ks += "Pause";
	  break;
	case Qt::Key_Print:
	  ks += "Print";
	  break;
	case Qt::Key_SysReq:
	  ks += "SysReq";
	  break;
	case Qt::Key_Home:
	  ks += "Home";
	  break;
	case Qt::Key_End:
	  ks += "End";
	  break;
	case Qt::Key_Left:
	  ks += "Left";
	  break;
	case Qt::Key_Up:
	  ks += "Up";
	  break;
	case Qt::Key_Right:
	  ks += "Right";
	  break;
	case Qt::Key_Down:
	  ks += "Down";
	  break;
	case Qt::Key_PageUp:
	  ks += "PageUp";
	  break;
	case Qt::Key_PageDown:
	  ks += "PageDown";
	  break;
	case Qt::Key_Super_L:
	  ks += "Super_L";
	  break;
	case Qt::Key_Super_R:
	  ks += "Super_R";
	  break;
	case Qt::Key_Menu:
	  ks += "Menu";
	  break;
	case Qt::Key_Hyper_L:
	  ks += "Hyper_L";
	  break;
	case Qt::Key_Hyper_R:
	  ks += "Hyper_R";
	  break;
	case Qt::Key_Help:
	  ks += "Help";
	  break;
	default:
	  ks += "<" + QString::number( key, 16 ) + ">";
	  break;
	}
    }
  return ks;
}
