/****************************************************************************
* keyboard_shortcuts.h
*
* This is a class for managing user customizable keyboard
* shortcuts. Compile with 'moc'.
*
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
* 'describeKey' function is derived from the OpieReader sources.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
****************************************************************************/

#ifndef KEYBOARD_SHORTCUTS_H
#define KEYBOARD_SHORTCUTS_H

#include <qtopia/config.h>
#include <qaction.h>
#include <qdialog.h>
#include <qlist.h>

class QListViewItem;
class KeyboardShortcutsDialog;
class QListView;

/**
 * KeyboardShortcuts manages and allows for user customization of
 * keyboard shortucts in Qtopia applications. Only actions that are
 * defined as QAction objects can be managed by this class.
 *
 * Typical usage scenario:
 *
 * 1. Create keyboardShortcuts object as a child of QMainWindow
 *
 * 2. Add all QActions that can have keyboard shortcuts to the
 *    keyboardShortcuts object
 *
 *    keyboardShortcuts->addAction( openFile );
 * 
 * 3. At program startup when loading preferences
 *
 *    keyboardShortcuts->load( conf );
 *
 *    to load keyboard shortcuts from the config file.
 *
 * 4. When 'Configure keyboard' is selected from the application menu
 *
 *    keyboardShortcuts->execConfig();
 *
 *    oo show modal dialog for changing keyboard binding.
 *
 * 5. At program exit or when the keyboard binding has changed
 *
 *    keyboardShortcuts->save( conf );
 *
 *    to store keyboard binding.
 *
 * There is no need to delete keyboardShortcuts object as it will be
 * deleted with QMainWindow object.
 */
class KeyboardShortcuts: public QObject {

  Q_OBJECT

  friend class KeyboardShortcutsDialog;
  
protected:
  QList<QAction> actionList;
  
public:

  /**
   * Creates KeyboardShortcuts object. Pass QMainWindow as a parent.
   */
  KeyboardShortcuts( QWidget *parent );

  /**
   * Before KeyboardShortcuts can manage key binding of QAction
   * object, QAction has to be added to KeyboardShortcuts object with
   * this method. The name of the action is taken from QAction::text()
   * and an optional pixmap from QAction()::pixmap(). To set default
   * key bindings, setAccel() for QAction objects.
   *
   * @param action Action that should have customizable keyboard shortcut
   */
  void addAction( QAction *action );

  /**
   * Loads a keyboard binding from the config file. Binding is stored
   * in the 'keyboard' group of the config file.
   *
   * @param conf an application configuration file
   */
  void load( Config *conf );

  /**
   * Stores a keyboard binding to the config file. Binding is stored
   * in the 'keyboard' group of the config file.
   * 
   * @param conf an application configuration file
   */
  void save( Config *conf );

  /**
   * Open a modal dialog for user customization of keyboard
   * bindings. Note that the new binding is not stored in the file
   * after call to this function. It is the responsibility of the
   * application to call 'save' method.
   */
  void execConfig();
};


/**
 * Returns a text description of the key code 'key'. Shift, Alt and
 * Ctrl modifiers are recognized. This function is used internally by
 * KeyboardShortcuts class.
 */
QString describeKey( int key );

// The class below is used only internally

class KeyboardShortcutsDialog: public QDialog {
  
Q_OBJECT

  friend class KeyboardShortcuts;

protected:

  QListViewItem *grabKeyFor;
  QListView *actionLV;
  bool ignoreReturn;
  bool modified;

public:

  KeyboardShortcutsDialog( QWidget *parent, KeyboardShortcuts *ks );
  
protected slots:
  void keyActionClicked( QListViewItem *item );

  void setKey();
  void removeKey();
  void cancelSetKey();

protected:
  void keyPressEvent( QKeyEvent *e );

};

#endif
